﻿using System;
using System.Collections;
using Alachisoft.NCache.Runtime.Caching;
using Alachisoft.NCache.Web.Caching;

// ===============================================================================
// Alachisoft (R) NCache Sample Code.
// NCache Tags sample class.
// ===============================================================================
// Copyright © Alachisoft.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
// ===============================================================================


namespace GroupsAndTags
{
	public class Tags
	{

		public static void RunTagsDemo()
		{
			try
			{
				Console.WriteLine();

				Cache cache;
				cache = NCache.InitializeCache("mypartitionedcache");
				cache.Clear();

				//Adding items with named Tags       
				//These tags are used to identify products who are ISOCertified, QCPassed and fall in ClassA
				Tag[] tagList = new Tag[3];
				tagList[0] = new Tag("ISOCertified");
				tagList[1] = new Tag("QCPassed");
				tagList[2] = new Tag("ClassA");

				//4 Items are added to the cache
				cache.Add("Product:MobilePhone1", "ProductID: XYZ", tagList);
				cache.Add("Product:MobilePhone2", "ProductID: ABC", tagList);
				cache.Add("Product:MobilePhone3", "ProductID: 123");
				cache.Add("Product:MobilePhone4", "ProductID: 456");

				//Retrieve items who are QCPassed
				Tag itemTag = new Tag("QCPassed");

				Hashtable items = cache.GetByTag(itemTag);
				if (items.Count > 0)
				{
					IEnumerator iter = items.Values.GetEnumerator();
					while (iter.MoveNext())
					{
						Console.WriteLine(iter.Current.ToString());
					}
				}

				//Get keys by tags
				//Here keys can be retrived from the cache via following three methods

				//Retrives keys by specified tag
				ICollection keysByTag = cache.GetKeysByTag(tagList[0]);

                if (keysByTag.Count > 0)
                {
                    IEnumerator iter = keysByTag.GetEnumerator();
                    while (iter.MoveNext())
                    {
                        Console.WriteLine(iter.Current.ToString());
                    }
                }

				//Retrieves those keys only where complete tagList matches
				ICollection keysByEntireTagList = cache.GetKeysByAllTags(tagList);

                if (keysByEntireTagList.Count > 0)
                {
                    IEnumerator iter = keysByEntireTagList.GetEnumerator();
                    while (iter.MoveNext())
                    {
                        Console.WriteLine(iter.Current.ToString());
                    }
                }

				//Retrieves keys where any item in tagList matches
				ICollection keysByAnyTagInList = cache.GetKeysByAnyTag(tagList);

                if (keysByAnyTagInList.Count > 0)
                {
                    IEnumerator iter = keysByAnyTagInList.GetEnumerator();
                    while (iter.MoveNext())
                    {
                        Console.WriteLine(iter.Current.ToString());
                    }
                }

				//Get Data by tags
				//Here values can be retrived from the cache via following three methods

				//Retrives values by specified tag
				Hashtable valuesByTag = cache.GetByTag(tagList[0]);

                if (valuesByTag.Count > 0)
                {
                    foreach (string key in valuesByTag.Keys)
                    {
                        Console.WriteLine("Fetched value by tag of the key : {0}", key);
                    }
                }

				//Retrieves those values only where complete tagList matches
				Hashtable valuesByEntireTagList = cache.GetByAllTags(tagList);

                if (valuesByEntireTagList.Count > 0)
                {
                    foreach (string key in valuesByTag.Keys)
                    {
                        Console.WriteLine("Fetched value by Entire tag-list of the key : {0}", key);
                    }
                }

				//Retrivies values where any item in tagList matches
			    Hashtable valuesByAnyTagInList = cache.GetByAnyTag(tagList);

                if (valuesByAnyTagInList.Count > 0)
                {
                    foreach (string key in valuesByTag.Keys)
                    {
                        Console.WriteLine("Fetched value by any tag of the key : {0}", key);
                    }
                }

				//Remove items from Cache by tags

				//Removes values by specified tag
				cache.RemoveByTag(tagList[0]);

				//Removes those values only where complete tagList matches
				cache.RemoveByAllTags(tagList);

				//Removes values where any item in tagList matches
				cache.RemoveByAnyTag(tagList);

				Console.WriteLine();

				//Must dispose cache
				cache.Dispose();

			}
			catch (Exception)
			{

			}
		}

	}

}